==> cloud-4.0.0-rc1.tar.gz: cloud/modules/cloud_service_providers/openstack/src/Form/OpenStackFloatingIpAssociateForm.php <==
<?php

namespace Drupal\openstack\Form;

use Drupal\openstack\Service\Ec2\OpenStackService as OpenStackEc2Service;
use Drupal\aws_cloud\Form\Ec2\ElasticIpAssociateForm;
use Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface;
use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\cloud\Service\EntityLinkRendererInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Plugin\CachedDiscoveryClearerInterface;
use Drupal\Core\Render\Markup;
use Drupal\openstack\Service\OpenStackServiceFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Associate Floating IP address.
 */
class OpenStackFloatingIpAssociateForm extends ElasticIpAssociateForm {
  /**
   * The OpenStack Service Factory.
   *
   * @var \Drupal\openstack\Service\OpenStackServiceFactoryInterface
   */
  protected $openStackServiceFactory;

  /**
   * OpenStackFloatingIpAssociateForm constructor.
   *
   * @param \Drupal\openstack\Service\OpenStackServiceFactoryInterface $openstack_service_factory
   *   Object for interfacing with OpenStack Service.
   * @param \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface $ec2_service
   *   The AWS Cloud EC2 service.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The messenger service.
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   *   The Entity Type Manager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheRender
   *   A cache backend interface instance.
   * @param \Drupal\Core\Plugin\CachedDiscoveryClearerInterface $plugin_cache_clearer
   *   A plugin cache clear instance.
   * @param \Drupal\cloud\Service\EntityLinkRendererInterface $entity_link_renderer
   *   The entity link render service.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager (CloudConfigPluginManager).
   */
  public function __construct(OpenStackServiceFactoryInterface $openstack_service_factory,
                              Ec2ServiceInterface $ec2_service,
                              EntityRepositoryInterface $entity_repository,
                              EntityTypeBundleInfoInterface $entity_type_bundle_info,
                              TimeInterface $time,
                              Messenger $messenger,
                              EntityTypeManager $entity_type_manager,
                              CacheBackendInterface $cacheRender,
                              CachedDiscoveryClearerInterface $plugin_cache_clearer,
                              EntityLinkRendererInterface $entity_link_renderer,
                              CloudConfigPluginManagerInterface $cloud_config_plugin_manager) {

    parent::__construct(
      $ec2_service,
      $entity_repository,
      $entity_type_bundle_info,
      $time,
      $messenger,
      $entity_type_manager,
      $cacheRender,
      $plugin_cache_clearer,
      $entity_link_renderer,
      $cloud_config_plugin_manager
    );

    $this->openStackServiceFactory = $openstack_service_factory;
  }

  /**
   * Dependency Injection.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   Instance of ContainerInterface.
   *
   * @return OpenStackFloatingIpAssociateForm
   *   return created object.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('openstack.factory'),
      $container->get('openstack.ec2'),
      $container->get('entity.repository'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time'),
      $container->get('messenger'),
      $container->get('entity_type.manager'),
      $container->get('cache.render'),
      $container->get('plugin.cache_clearer'),
      $container->get('entity.link_renderer'),
      $container->get('plugin.manager.cloud_config_plugin')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = '') {
    $entity = $this->entity;

    // Switch OpenStack EC2 or REST service based on $cloud_context.
    if (!$this->updateEntityBuildForm($entity)) {
      return $this->redirectUrl;
    }

    $openstack_ec2_service_type = $this->openStackServiceFactory->isEc2ServiceType('cloud_config', $cloud_context);

    if (!empty($openstack_ec2_service_type)) {
      return $form = parent::buildForm($form, $form_state, $cloud_context);
    }

    if (empty($openstack_ec2_service_type)) {
      if ($entity->getAssociationId() !== NULL) {
        $form['error'] = [
          '#markup' => '<div>'
          . $this->t('@label is already associated.', [
            '@label' => $entity->getEntityType()->getSingularLabel(),
          ])
          . '</div>',
        ];
        unset($form['actions']['submit']);
      }
      else {
        $form = parent::buildForm($form, $form_state, $cloud_context);
        // DO NOT change isset() to !empty() since When we change from isset to
        // !empty(), getting an AJAX error while changing Private IP like
        // An illegal choice.
        // Ajax support: Look at the instance value, and rebuild the private_ip
        // options.
        $instance = $form_state->getUserInput()['instance_id'] ?? '';
        if (isset($instance)) {
          $form['instance_ip_container']['instance_private_ip']['#options'] = $instance !== '-1'
            ? $this->getPrivateIps($instance)
            : ['-1' => $this->t('Select a private IP.')];
        }
      }
    }

    return $form;
  }

  /**
   * Submit handler.
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {

    $entity = $this->entity;

    // Switch OpenStack EC2 or REST service based on $entity->getCloudContext().
    $this->ec2Service = $this->openStackServiceFactory->get($entity->getCloudContext());

    if ($this->ec2Service instanceof OpenStackEc2Service) {
      parent::submitForm($form, $form_state);
      return;
    }

    // Get module name.
    $module_name = $this->getModuleName($entity);

    // Determine if an IP address is attaching to instance or network_interface.
    if ($form_state->getValue('resource_type') === 'instance') {

      $instance_entity_id = $form_state->getValue('instance_id');
      $private_ip = $form_state->getValue('instance_private_ip');

      if ($instance_entity_id !== -1) {

        $instance = $this->entityTypeManager
          ->getStorage("{$module_name}_instance")
          ->loadByProperties(['id' => $instance_entity_id]);

        if (empty($instance)) {
          $this->messenger->addError(
            $this->t('Unable to load the instance %instance_entity_id.', [
              '%instance_entity_id' => $instance_entity_id,
            ]
          ));

          $form_state->setRedirect("entity.{$entity->getEntityTypeId()}.canonical", [
            'cloud_context' => $entity->getCloudContext(),
            "{$entity->getEntityTypeId()}" => $entity->id(),
          ]);
          return;
        }

        $instance = array_shift($instance);
        $instance_id = $instance->getInstanceId();

        $instance_link = $this->entityLinkRenderer->renderViewElement(
        $instance_id,
        "{$module_name}_instance",
        'instance_id',
        [],
        $instance->getName() !== $instance->getInstanceId()
          ? $this->t('@instance_name (@instance_id)', [
            '@instance_name' => $instance->getName(),
            '@instance_id' => $instance->getInstanceId(),
          ])
          : $instance_id
        );

        $entity->setInstanceId($instance->getInstanceId());
        $entity->save();

        // Associate instance.
        $result = $this->ec2Service->associateAddress([
          'FloatingIpId' => $entity->getFloatingIpId(),
          'PortId' => $private_ip,
        ]);

        if (!empty($result)) {
          $message = $this->t("@label @ip_address associated with @private_ip for instance: @instance_id", [
            '@ip_address' => $entity->getPublicIp(),
            '@private_ip' => $private_ip,
            '@instance_id' => Markup::create($instance_link['#markup']),
            '@label' => $entity->getEntityType()->getSingularLabel(),
          ]);

          $this->updateFloatingIpEntity($message);
          $this->clearCacheValues($entity->getCacheTags());
        }
        else {
          $this->messenger->addError($this->t('Unable to associate @label.', [
            '@label' => $entity->getEntityType()->getSingularLabel(),
          ]));
        }
      }
      else {
        $this->messenger->addError($this->t('Unable to load instance ID. No association performed.'));
      }
    }
    else {
      $network_interface_id = $form_state->getValue('network_interface_id');
      $network_private_ip = $form_state->getValue('network_private_ip');

      if ($network_interface_id !== -1) {

        $network_interface = $this->entityTypeManager
          ->getStorage("{$module_name}_network_interface")
          ->loadByProperties(['id' => $network_interface_id]);

        if (count($network_interface) === 1) {
          $network_interface = array_shift($network_interface);
        }

        // Associate network interface.
        $result = $this->ec2Service->associateAddress([
          'FloatingIpId' => $entity->getFloatingIpId(),
          'PortId' => $network_private_ip,
        ]);

        if (!empty($result)) {
          $message = $this->t('@label @ip_address associated with @private_ip for network interface: @network_interface_id', [
            '@ip_address' => $entity->getPublicIp(),
            '@network_interface_id' => $network_interface->getNetworkInterfaceId(),
            '@private_ip' => $network_private_ip,
            '@label' => $entity->getEntityType()->getSingularLabel(),
          ]);

          $this->updateFloatingIpEntity($message);
          $this->clearCacheValues($entity->getCacheTags());
        }
        else {
          $this->messenger->addError($this->t("Unable to associate @label.", [
            '@label' => $entity->getEntityType()->getSingularLabel(),
          ]));
        }
      }
      else {
        $this->messenger->addError($this->t('Unable to load instance ID. No association performed.'));
      }
    }

    // Refresh the instances.
    $this->ec2Service->updateInstances();

    $form_state->setRedirect("entity.{$entity->getEntityTypeId()}.canonical", [
      'cloud_context' => $entity->getCloudContext(),
      "{$entity->getEntityTypeId()}" => $entity->id(),
    ]);
  }

  /**
   * Helper function that loads all the private IPs for an instance.
   *
   * @param int $instance_id
   *   The instance ID.
   *
   * @return array
   *   An array of IP addresses.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getPrivateIps($instance_id): array {
    $entity = $this->entity;

    // Get module name.
    $module_name = $this->getModuleName($entity);

    $instance = $this->entityTypeManager
      ->getStorage("{$module_name}_instance")
      ->loadByProperties(['id' => $instance_id]);

    if (count($instance) === 1) {
      $instance = array_shift($instance);
    }

    $ips = !empty($instance) ? explode(', ', $instance->getPrivateSecondaryIp()) : [];
    $private_ips = [];
    foreach ($ips ?: [] as $ip) {
      $private_ips[$ip] = $ip;
    }

    return $private_ips;
  }

  /**
   * Helper function to update the current openstack_elastic_ip entity.
   *
   * @param string $message
   *   Message to display to use.
   */
  private function updateFloatingIpEntity($message): void {

    // Update the following entities from OpenStack.
    $this->ec2Service->updateFloatingIps();
    $this->ec2Service->updateNetworkInterfaces();
    $this->ec2Service->updateInstances();
    $this->messenger->addStatus($message);
  }

}

==> cloud-5.0.0-rc1.tar.gz: cloud/modules/cloud_service_providers/openstack/src/Form/OpenStackFloatingIpAssociateForm.php <==
<?php

namespace Drupal\openstack\Form;

use Drupal\aws_cloud\Form\Ec2\ElasticIpAssociateForm;
use Drupal\aws_cloud\Service\AwsCloud\AwsCloudOperationsServiceInterface;
use Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface;
use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\cloud\Service\EntityLinkRendererInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Plugin\CachedDiscoveryClearerInterface;
use Drupal\openstack\Service\OpenStackOperationsServiceInterface;
use Drupal\openstack\Service\OpenStackServiceFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Associate Floating IP address.
 */
class OpenStackFloatingIpAssociateForm extends ElasticIpAssociateForm {
  /**
   * The OpenStack Service Factory.
   *
   * @var \Drupal\openstack\Service\OpenStackServiceFactoryInterface
   */
  protected $openStackServiceFactory;

  /**
   * The OpenStack operations Service.
   *
   * @var \Drupal\openstack\Service\OpenStackOperationsServiceInterface
   */
  protected $openStackOperationsService;

  /**
   * OpenStackFloatingIpAssociateForm constructor.
   *
   * @param \Drupal\aws_cloud\Service\AwsCloud\AwsCloudOperationsServiceInterface $aws_cloud_operations_service
   *   The AWS Cloud Operations service.
   * @param \Drupal\openstack\Service\OpenStackServiceFactoryInterface $openstack_service_factory
   *   Object for interfacing with OpenStack Service.
   * @param \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface $ec2_service
   *   The AWS Cloud EC2 service.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The messenger service.
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   *   The Entity Type Manager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheRender
   *   A cache backend interface instance.
   * @param \Drupal\Core\Plugin\CachedDiscoveryClearerInterface $plugin_cache_clearer
   *   A plugin cache clear instance.
   * @param \Drupal\cloud\Service\EntityLinkRendererInterface $entity_link_renderer
   *   The entity link render service.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager (CloudConfigPluginManager).
   * @param \Drupal\openstack\Service\OpenStackOperationsServiceInterface $openstack_operations_service
   *   The OpenStack Operations service.
   */
  public function __construct(
    AwsCloudOperationsServiceInterface $aws_cloud_operations_service,
    OpenStackServiceFactoryInterface $openstack_service_factory,
    Ec2ServiceInterface $ec2_service,
    EntityRepositoryInterface $entity_repository,
    EntityTypeBundleInfoInterface $entity_type_bundle_info,
    TimeInterface $time,
    Messenger $messenger,
    EntityTypeManager $entity_type_manager,
    CacheBackendInterface $cacheRender,
    CachedDiscoveryClearerInterface $plugin_cache_clearer,
    EntityLinkRendererInterface $entity_link_renderer,
    CloudConfigPluginManagerInterface $cloud_config_plugin_manager,
    OpenStackOperationsServiceInterface $openstack_operations_service
  ) {

    parent::__construct(
      $aws_cloud_operations_service,
      $ec2_service,
      $entity_repository,
      $entity_type_bundle_info,
      $time,
      $messenger,
      $entity_type_manager,
      $cacheRender,
      $plugin_cache_clearer,
      $entity_link_renderer,
      $cloud_config_plugin_manager
    );

    $this->openStackServiceFactory = $openstack_service_factory;
    $this->openStackOperationsService = $openstack_operations_service;
  }

  /**
   * Dependency Injection.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   Instance of ContainerInterface.
   *
   * @return OpenStackFloatingIpAssociateForm
   *   return created object.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('aws_cloud.operations'),
      $container->get('openstack.factory'),
      $container->get('openstack.ec2'),
      $container->get('entity.repository'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time'),
      $container->get('messenger'),
      $container->get('entity_type.manager'),
      $container->get('cache.render'),
      $container->get('plugin.cache_clearer'),
      $container->get('entity.link_renderer'),
      $container->get('plugin.manager.cloud_config_plugin'),
      $container->get('openstack.operations')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = '') {
    /** @var \Drupal\aws_cloud\Entity\Ec2\ElasticIpInterface $entity */
    $entity = $this->entity;

    // Switch OpenStack EC2 or REST service based on $cloud_context.
    if (!$this->updateEntityBuildForm($entity)) {
      return $this->redirectUrl;
    }

    $openstack_ec2_service_type = $this->openStackServiceFactory->isEc2ServiceType('cloud_config', $cloud_context);

    if (!empty($openstack_ec2_service_type)) {
      return $form = parent::buildForm($form, $form_state, $cloud_context);
    }

    if (empty($openstack_ec2_service_type)) {
      if ($entity->getAssociationId() !== NULL) {
        $form['error'] = [
          '#markup' => '<div>'
          . $this->t('@label is already associated.', [
            '@label' => $entity->getEntityType()->getSingularLabel(),
          ])
          . '</div>',
        ];
        unset($form['actions']['submit']);
      }
      else {
        $form = parent::buildForm($form, $form_state, $cloud_context);
        // DO NOT change isset() to !empty() since When we change from isset to
        // !empty(), getting an AJAX error while changing Private IP like
        // An illegal choice.
        // Ajax support: Look at the instance value, and rebuild the private_ip
        // options.
        $instance = $form_state->getValue('instance_id');
        $ips = isset($instance) && $instance !== '-1'
          ? $this->openStackOperationsService->getPrivateIps($entity, $instance)
          : [];

        $form['instance_ip_container']['instance_private_ip']['#options'] = !empty($ips)
          ? $ips
          : ['-1' => $this->t('Select a private IP.')];
      }
    }
    return $form;
  }

  /**
   * Submit handler.
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->openStackOperationsService->associateOpenStackFloatingIp($this->entity, $form, $form_state);
  }

}
